#include <stdio.h>
#include <string.h>
#include <NDS/nds.h>
#include <ds_misc.h>
#include "shared.h"
#include "7.h"

void bios_firmware_read(u32 srcaddr,u8 *dstaddr,u32 size);
void bios_SPI_write(int device,int data,int select);

int fifo_send(u32 data) {
    if (REG_FIFO_CNT & FIFO_CNT_ERR) {
		REG_FIFO_CNT = FIFO_CNT_SEND_CLEAR | FIFO_CNT_SEND_IRQ | FIFO_CNT_RECV_IRQ | FIFO_CNT_ENABLE | FIFO_CNT_ERR;
		return FAIL_SEND_ERR;
    }
	if(REG_FIFO_CNT&FIFO_CNT_SEND_FULL)
		return FAIL_SEND_FULL;
	REG_FIFO_SEND = data;
	return SUCCESS;
}

int line=0;
void text(char *s) {
	u16 *p;
	p=(u16*)(0x2000000+line*64);
	while(*s)
		*(p++)=*(s++);
	while((u32)p&0x3f)
		*(p++)=' ';
	fifo_send(line++);
}

void textN(char *s,int line) {
	u16 *p;
	p=(u16*)(0x2000000+line*64);
	while(*s)
		*(p++)=*(s++);
	while((u32)p&0x3f)
		*(p++)=' ';
	fifo_send(line);
}

u8 writeread(u8 data) {
	while (SERIAL_CR & SERIAL_BUSY);
	SERIAL_DATA = data;
	while (SERIAL_CR & SERIAL_BUSY);
	return SERIAL_DATA;
	
}

u8 writePM(u8 channel, u8 data) {
	u8 ret;
	SERIAL_CR = SPI_EN|SPI_SEL|SPI_POWER|2;
	writeread(channel);
	SERIAL_CR = SPI_EN|SPI_POWER|2;
	ret=writeread(data);	
	SERIAL_CR = 0;
	return ret;
}
#define readPM(chan) writePM((chan)|0x80,0)

void flash(u8 *src,u32 dst) {
	int i;
	
	//write enable
	SERIAL_CR = SPI_EN|SPI_SEL|SPI_FW;
	writeread(6);
	SERIAL_CR = 0;
	
	//Wait for Write Enable Latch to be set
	SERIAL_CR = SPI_EN|SPI_SEL|SPI_FW;
	writeread(5);
	while((writeread(0)&0x02)==0); //Write Enable Latch
	SERIAL_CR = 0;
	
	//page write
	SERIAL_CR = SPI_EN|SPI_SEL|SPI_FW;
	writeread(0x0A);
	writeread((dst&0xff0000)>>16);
	writeread((dst&0xff00)>>8);
	writeread(0);
	for (i=0; i<256; i++) {
		writeread(src[i]);
	}
	SERIAL_CR=0;
	// wait programming to finish
	SERIAL_CR = SPI_EN|SPI_SEL|SPI_FW;
	writeread(0x05);
	while(writeread(0)&0x01);	// WIP (Write In Progress)
	SERIAL_CR = 0;
}

int flash_verify(u8 *new,u8 *old,u32 fwaddr) {
	int i,cmp;	

	cmp=0;
	for(i=0;i<256;i++) {
		cmp|=(old[i]!=new[i]);
	}
	if(!cmp) return 1;	//already the same, don't bother flashing
	
	flash(new,fwaddr);
	bios_firmware_read(fwaddr,old,256);

	cmp=0;
	for(i=0;i<256;i++) {
		cmp|=(old[i]!=new[i]);
	}
	return !cmp;
}

void initialize_DS() {
	u8 lite;
	writePM(0,PM_BOTTOMLIGHT|PM_SOUND);

	lite=readPM(4);			//make sure lighting is set (in case of failsafe boot)
	if(lite&0x40) {
		lite&=0xfc;
		lite|=2;
		writePM(4,lite);
	}

	soundinit();

}

void VBlankWait() {			//interrupts don't work for some fucking reason
	while(DISP_SR&DISP_IN_VBLANK);
	while(!(DISP_SR&DISP_IN_VBLANK));
}

#define KEY_X 0x400
#define KEY_Y 0x800
int getkey() {
	static int oldkey=-1;
	int k,x;
	k=READ_KEYS;
	k|=(~XKEYS&3)<<10;
	x=(oldkey^k)&k;
	oldkey=k;
	return x;
}

void smalldelay() {
	int i;
	for(i=0;i<5;i++)
		VBlankWait();
}

void stop() {
//	fifo_send(99);
	while(1) swiSleep();
}

u16 keysequence[]={KEY_X,KEY_B,KEY_X,KEY_B};

void arm7main() {
	int lite;
	char s[32];
	char origstr[32];
	u8 *oldFW=(u8*)0x2300000;
	u8 *newFW;
	u16 k;
	int i,error;
	int seq;
	u16 crc;
	u16 flashme_version;
	int nds_version;
	int sfx_timeout=0;

	PROGRESS=0;
	HEARTBEAT=0;
	initialize_DS();
		
	fifo_init();
	FIFO_INTR=0x700;
	while((FIFO_INTR&15)!=9);	//wait for arm9 ready

	lite=(readPM(4)&0x40)==0x40;
	
//dump current firmware

	bios_firmware_read(0,oldFW,0x40000);

//check version

	flashme_version=*(u16*)(oldFW+0x17c);
	crc=*(u16*)(oldFW+0x17e);
	error=0;
	if(crc==0xffff) {
		text("----------   WARNING   ---------");
		text("Your firmware doesn't appear to");
		text("be patched.");
		crc=*(u16*)(oldFW+6);
		error=1;
	} else if(flashme_version!=0x0001 && flashme_version!=0x0002) {
		text("----------   WARNING   ---------");
		text("Unknown firmware found.");
		error=1;
	}

	//find original FW ver
	nds_version=-1;
	for(i=0;i<VERSIONS;i++) {
		if(crc==FIRMWARES[i][3]) {
			nds_version=i;
			break;
		}
	}
	if(nds_version<0) {
		text("----------   WARNING   ---------");
		text("Your original firmware version");
		text("cannot be determined.  If you");
		text("proceed, a different version");
		text("from the original may be");
		text("installed instead.");
		error=1;
		if(!lite)
			nds_version=2;	//default to v3
		else
			nds_version=5;	//lite defaults to v6
		strcpy(origstr,"?");
	} else {
		sprintf(origstr,"NDS-V%i",nds_version+1);
	}
	if(error) {
		for(i=0;i<60*3;i++)
			VBlankWait();
		text("");
		text("Push START to continue anyway.");
		do { k=getkey(); } while(k!=KEY_START);
		line=0;
	}

//print info

	text("FIRMWARE RESTORE UTILITY");
	text("--------------------------------");
	text("This tool removes any firmware");
	text("updates, including failsafe");
	text("recovery code. If power is lost");
	text("before flashing completes, your");
	text("NDS may be permanently damaged.");
	text("You should plug in a power");
	text("source before proceeding.");

//verify firmware integrity
	for(i=0;i<VERSIONS;i++) {
		newFW=(u8*)(FIRMWARES[i]);
		crc=swiCRC(0xffff,(u32*)newFW,0x3fdfe);
		if(crc!=*(u16*)(newFW+0x3fdfe)) {
			text("NDS file is corrupt.");
			sprintf(s,"%x %x %x %x",i+1,(u32)newFW,*(u32*)newFW,*(u32*)(newFW+0x3fdfc));//crc);
			text(s);
			writePM(0,PM_BOTTOMLIGHT|PM_TOPLIGHT|PM_SOUND);
			fifo_send(99);
			stop();
		}
		*(u16*)(newFW+0x3fdfe)=0xffff;
	}
	
//prompt user

	textN("--------------------------------",22);
	text("Push X B X B to continue.");
	text("");

change_ver:
	sprintf(s,"Orig:%s  New:NDS-V%i (0x%04X)",origstr,nds_version+1,FIRMWARES[nds_version][3]);
	textN(s,23);

	seq=0;
	do {
		VBlankWait();
		k=getkey();
		if(k) {
			if(k==keysequence[seq])
				seq++;
			else
				seq=0;
			if(k==KEY_SELECT) {
				nds_version--;
				if(nds_version<0)
					nds_version=VERSIONS-1;
				goto change_ver;
			}
		}
	} while(seq<4);	
	text("If progress stops, the SL1");
	text("terminal needs to be shorted.");
	text("Please keep it shorted until");
	text("flashing is finished.");
	text("");
	text("Progress:   0%");

//keep header info

	newFW=(u8*)(FIRMWARES[nds_version]);

	for(i=0x28;i<0x170;i++)
		newFW[i]=oldFW[i];

//start the magic

	if(flashme_version==0x0001)
		i=0x3fd00;			//i=starting page number
	else //if(flashme_version==0x0002)
		i=0x3f700;
	
	do {
		if(!flash_verify(newFW+i,oldFW+i,i)) {
			smalldelay();
			HEARTBEAT++;
			if(++sfx_timeout>6) {
				playsound(SFX_STEP);
				sfx_timeout=0;
			}				
		} else {
			if(++sfx_timeout>3) {
				playsound(SFX_STEP);
				sfx_timeout=0;
			}				
			HEARTBEAT=0;
			i-=0x100;
		}
		PROGRESS=0x3fd00-i;
	} while(i>=0);

//all done
	smalldelay(); 	playsound(SFX_BARREL);
	smalldelay();	playsound(SFX_STEP);
	smalldelay(); 	playsound(SFX_JUMP);
	smalldelay(); 	playsound(SFX_BARREL);
	smalldelay();	playsound(SFX_STEP);
	smalldelay(); 	playsound(SFX_JUMP);
	smalldelay(); 	playsound(SFX_BARREL);

	text("");
	text("Firmware flashing completed");
	text("successfully.");
	stop();
}

