#include <NDS/NDS.h>
#include <string.h>
#include <ds_misc.h>
#include "shared.h"

extern u16 font;
extern u16 fontpal;
extern u8 firmware1[]; 
extern u8 firmware2[]; 
extern u8 firmware3[];
extern u8 firmware4[];
extern u8 firmware5[];
extern u8 firmware6[];
extern u8 firmware7[];

#define vramSetBanks(a,b,c,d) (VRAM_CR=0x80808080|(a)|(b<<8)|(c<<16)|(d<<24))
void gfx_init() {
	int i;
	u16 *p1=(u16*)0x6000000;
	u16 *p2=(u16*)0x2000000;
	
	POWER_CR=3;				//POWER LCDs, POWER MAIN 2D
	VRAM_CR=0x00000081;		//A->6000000
	
	for(i=0;i<32*24;i++) {
		*(p1++)=' ';	//screen clear
		*(p2++)=' ';	//screen buffer clear
	}
	dmaCopyHalfWords(3,&font,(u16*)(CHAR_BASE_BLOCK(1)+32*32),1024*3);
	dmaCopyHalfWords(3,&fontpal,(u16*)BG_PALETTE,64);
	
	DISPLAY_CR=MODE_0_2D|DISPLAY_BG0_ACTIVE;
	BG0_CR=BG_TILE_BASE(1);
	BG0_X0=0;
	BG0_Y0=0;
	BLEND_CR=0;
}

int fifo_recv (u32 *data_buf) {
    if (REG_FIFO_CNT & FIFO_CNT_ERR ) {
		REG_FIFO_CNT = FIFO_CNT_SEND_CLEAR | FIFO_CNT_SEND_IRQ | FIFO_CNT_RECV_IRQ | FIFO_CNT_ENABLE |FIFO_CNT_ERR;
		return FAIL_RECV_ERR;
    }
    if(REG_FIFO_CNT & FIFO_CNT_RECV_EMPTY)
		return FAIL_RECV_EMPTY;
	*data_buf = REG_FIFO_RECV;
	return SUCCESS;
}

int finished=0;
void FifoInterrupt() {
	int i;
	u16 *p1,*p2;
	u32 incoming;
	int res;
	do {
		res=fifo_recv(&incoming);
		if(res==SUCCESS && incoming<24) {
			p1=(u16*)(0x2000000+incoming*64);
			p2=(u16*)((u32)p1|0x6000000);
			for(i=0;i<32;i++) p2[i]=p1[i];
		}
		if(incoming==99) finished=1;
	} while(res==SUCCESS);
}

static void VblankInterrupt() {
	u16 *p;
	u32 d=PROGRESS;
	if(!d) return;
	p=(u16*)(SCREEN_BASE_BLOCK(0)+16*64+20);
	d=d/(0x3fe00/100);
	if(!d) d++;
	if(d>99) p[0]='1';
	if(d>9) p[1]='0'+(d/10)%10;
	p[2]='0'+d%10;
	p[3]=HEARTBEAT&1?' ':'%';
}

static void interrupthandler() {
	u32 flags=IF&IE;
	VBLANK_INTR_WAIT_FLAGS|=flags;
	if(flags&IRQ_VBLANK)
		VblankInterrupt();
	if(flags&IRQ_FIFO_FULL)
		FifoInterrupt();
	IF=flags;				//irq ack
}

void memdump(void);
void arm9main(int mode) {
	IME=0;
	DISP_SR=DISP_VBLANK_IRQ;
	IRQ_HANDLER=interrupthandler;
	IE=IRQ_VBLANK|IRQ_FIFO_FULL;
	IF=~0;
	IME=IME_ENABLED;

	FIRMWARE_1=firmware1;
	FIRMWARE_2=firmware2;
	FIRMWARE_3=firmware3;
	FIRMWARE_4=firmware4;
	FIRMWARE_5=firmware5;
	FIRMWARE_6=firmware6;
	FIRMWARE_7=firmware7;

	gfx_init();
	fifo_init();
	FIFO_INTR=0x900;
	while((FIFO_INTR&15)!=7);	//wait for arm9 ready

	while(!finished) swiSleep();
	memdump();
}
